﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using System.Windows.Forms;
using STFXPdfDotNET;
using CTNet;
using System.IO;

namespace CalcHelper
{
    static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>

        /*
         * Verifies launch arguments returning true or false.
         * 
         * Called From: Main(string[] args).
         */
        public static bool ProcessArgs(string[] args)
        {
            bool ret = false;

            //MessageBox.Show(string.Format("  args[0]:\n\n{0}", args[0]), "args[0]", MessageBoxButtons.OK, MessageBoxIcon.Information);
            //MessageBox.Show(string.Format("  args[1]:\n\n{0}", args[1]), "args[1]", MessageBoxButtons.OK, MessageBoxIcon.Information);
            //MessageBox.Show(string.Format("  args.Length:\n\n{0}", args.Length), "args.Length", MessageBoxButtons.OK, MessageBoxIcon.Information);

            /*
             * Caller has ensured args.Length > 0.
             */
            if (args.Length == 2)
            {
                if (args[1] == "VSC")
                {
                    /*
                     * Starting up from Visual Studio Code.
                     * Verify that Calc Helper can display the Visual Studio Code document to the user.
                     */
                    //MessageBox.Show(portFile, "Portfile:", MessageBoxButtons.OK, MessageBoxIcon.Information);

                    Form1.vscFileInfo = new VSCFileInfo();
                    Form1.vscFileInfo.VerifyFileInfo(args[0]);

                    switch (Form1.vscFileInfo.verifyResult)
                    {
                        // Probably unlikely...
                        case (verifyFile.nullEmptyPath):
                            MessageBox.Show("  Calc Helper launched with arg[0] either null or empty:\n\n  Launch will be aborted.", "Calc Helper Launch Error:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                            break;

                        // Probably unlikely...
                        case (verifyFile.nullEmptyFileName):
                            MessageBox.Show("  Calc Helper launched with a null or empty file name.\n\n  Launch will be aborted.", "Calc Helper Launch Error:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                            break;

                        // Probably unlikely...
                        case (verifyFile.nullWorkFolder):
                            MessageBox.Show("  Calc Helper launched with a null folder.\n\n  Launch will be aborted.", "Calc Helper Launch Error:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                            break;

                        case (verifyFile.notDocFile):
                            MessageBox.Show(string.Format("  Calc Helper launched from an editor file not belonging to a\n  document:\n\n  {0}\n\n  Launch will be aborted.", Form1.vscFileInfo.workFolderTmp), "Calc Helper Launch Error:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                            break;

                        case (verifyFile.noPortFile):
                            MessageBox.Show(string.Format("  The .calctools port number file can not be located:\n\n  {0}\n\n  Launch will be aborted.", Form1.vscFileInfo.portFileTmp), "Calc Helper Launch Error:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                            break;

                        case (verifyFile.noZipFile):

                            // Need to ask user where it is:
                            Form1.vscFileInfo.zipFileTmp = FindDocZipFile(Form1.vscFileInfo.zipFileNameTmp);

                            if (string.IsNullOrEmpty(Form1.vscFileInfo.zipFileTmp))
                            {
                                //MessageBox.Show(string.Format("  The document's zip file can not be located:\n\n  {0}\n\n  Launch will be aborted.", Form1.vscFileInfo.zipFileTmp), "Calc Helper Launch Error:", MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxOptions.DefaultDesktopOnly);
                            }
                            else
                            {
                                ret = true;
                            }

                            break;

                        case (verifyFile.exception):
                            MessageBox.Show("  An exception has occurred during file verification.\n\n  Launch will be aborted.", "Calc Helper Launch Error:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                            break;

                        case (verifyFile.fileOk):
                            ret = true;
                            break;

                        default:
                            MessageBox.Show("  An exception has occurred during file verification.\n\n  Launch will be aborted.", "Calc Helper Launch Error:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                            break;
                    }
                }
                else
                {
                    MessageBox.Show(string.Format("  Calc Helper launched with incorrect second argument.\n\n  Expected 'VSC' for Visual Studio Code but received\n  '{0}'\n\n  Launch will be aborted.", args[1]), "Calc Helper Launch Error:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else
            {
                MessageBox.Show(string.Format("  Calc Helper launched with {0} arguments where 2 were expected:\n\n  Launch will be aborted.", args.Length), "Calc Helper Launch Error:", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            return ret;
        }

        public static string FindDocZipFile(string zipFileName)
        {
            string location = null;
            string msg = "";
            string userZipFileName = "";

            bool getZip = true;

            OpenFileDialog openFileDialog = new System.Windows.Forms.OpenFileDialog();

            openFileDialog.Title = string.Format("Calc Helper needs your help in locating: {0}", zipFileName);
            openFileDialog.InitialDirectory = CalcHelper.Form1.pdfCurDir;
            openFileDialog.FileName = zipFileName;
            openFileDialog.DefaultExt = "zip";
            openFileDialog.Filter = string.Format("|{0}", zipFileName);

            try
            {
                do
                {
                    DialogResult result = openFileDialog.ShowDialog();

                    if (result == DialogResult.OK)
                    {
                        userZipFileName = Path.GetFileName(openFileDialog.FileName);

                        if (userZipFileName.ToUpper() == zipFileName.ToUpper())
                        {
                            location = openFileDialog.FileName;

                            getZip = false;
                        }
                        else
                        {
                            openFileDialog.FileName = zipFileName;

                            msg = string.Format("Calc Helper needs to know where this zip file is located:\n\n {0}", zipFileName);
                            MessageBox.Show(msg, "Incorrect File Name:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                    }
                    // Cancel button was pressed.
                    else if (result == DialogResult.Cancel)
                    {
                        getZip = false;
                    }
                } while (getZip == true);
            }
            catch (Exception ex)
            {
                location = null;

                MessageBox.Show(ex.ToString(), "FindDocZipFile():", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            if (string.IsNullOrEmpty(location))
            {
                MessageBox.Show(string.Format("  The document's zip file can not be located:\n\n  {0}\n\n  Launch will be aborted.", Form1.vscFileInfo.zipFileTmp), "Calc Helper Launch Error:", MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1, MessageBoxOptions.DefaultDesktopOnly);
            }

            return location;
        }

        [STAThread]

        static void Main(string[] args)
        {
            // Original code - start: Down below is how the other tools are coded re ProcessArgs().
            //Application.EnableVisualStyles();
            //Application.SetCompatibleTextRenderingDefault(false);
            //Application.Run(new Form1());
            // Original code - stop:

            //MessageBox.Show(string.Format("args length: {0}", args.Length));

            bool runForm = false;

            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);

            if (args.Length > 0)
            {
                // We return true if the args check out ok.
                runForm = ProcessArgs(args);
            }
            else
            {
                MessageBox.Show("  Calc Helper only launches from Visual Studio Code.\n\n  Launch will be aborted.", "Calc Helper Launch Error:", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            try
            {
                if (runForm == true)
                {
                    Application.Run(new Form1(args));
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }
}
